@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2006-2007 Collabora Limited
  * Copyright (C) 2006-2007 Nokia Corporation
+ * Copyright (C) 2011      Debarshi Ray <rishi@gnu.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -28,6 +29,7 @@
 #define DBUS_API_SUBJECT_TO_CHANGE
 #include <dbus/dbus-glib.h>
 
+#include <telepathy-glib/dbus.h>
 #include <telepathy-glib/enums.h>
 #include <telepathy-glib/errors.h>
 #include <telepathy-glib/interfaces.h>
@@ -36,6 +38,7 @@
 #include <telepathy-glib/channel-manager.h>
 
 #define IDLE_DEBUG_FLAG IDLE_DEBUG_CONNECTION
+#include "idle-contact-info.h"
 #include "idle-ctcp.h"
 #include "idle-debug.h"
 #include "idle-handles.h"
@@ -54,6 +57,8 @@
  * be subject to this mechanism.
  */
 
+#define DEFAULT_KEEPALIVE_INTERVAL 30 /* sec */
+
 #define MSG_QUEUE_UNLOAD_AT_A_TIME 1
 #define MSG_QUEUE_TIMEOUT 2
 
@@ -75,6 +80,7 @@ static void _renaming_iface_init(gpointer, gpointer);
 
 G_DEFINE_TYPE_WITH_CODE(IdleConnection, idle_connection, TP_TYPE_BASE_CONNECTION,
 		G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_CONNECTION_INTERFACE_ALIASING, _aliasing_iface_init);
+		G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_CONNECTION_INTERFACE_CONTACT_INFO, idle_contact_info_iface_init);
 		G_IMPLEMENT_INTERFACE(IDLE_TYPE_SVC_CONNECTION_INTERFACE_RENAMING, _renaming_iface_init));
 
 typedef struct _IdleOutputPendingMsg IdleOutputPendingMsg;
@@ -121,6 +127,7 @@ enum {
 	PROP_REALNAME,
 	PROP_USERNAME,
 	PROP_CHARSET,
+	PROP_KEEPALIVE_INTERVAL,
 	PROP_QUITMESSAGE,
 	PROP_USE_SSL,
 	PROP_PASSWORD_PROMPT,
@@ -145,6 +152,7 @@ struct _IdleConnectionPrivate {
 	char *realname;
 	char *username;
 	char *charset;
+	guint keepalive_interval;
 	char *quit_message;
 	gboolean use_ssl;
 	gboolean password_prompt;
@@ -161,6 +169,9 @@ struct _IdleConnectionPrivate {
 	/* UNIX time the last message was sent on */
 	time_t last_msg_sent;
 
+	/* GSource id for keep alive message timeout */
+	guint keepalive_timeout;
+
 	/* GSource id for message queue unloading timeout */
 	guint msg_queue_timeout;
 
@@ -188,6 +199,7 @@ static void _iface_disconnected(TpBaseConnection *self);
 static void _iface_shut_down(TpBaseConnection *self);
 static gboolean _iface_start_connecting(TpBaseConnection *self, GError **error);
 
+static IdleParserHandlerResult _error_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data);
 static IdleParserHandlerResult _erroneous_nickname_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data);
 static IdleParserHandlerResult _nick_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data);
 static IdleParserHandlerResult _nickname_in_use_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data);
@@ -206,6 +218,8 @@ static void connection_disconnect_cb(IdleConnection *conn, TpConnectionStatusRea
 static gboolean idle_connection_hton(IdleConnection *obj, const gchar *input, gchar **output, GError **_error);
 static void idle_connection_ntoh(IdleConnection *obj, const gchar *input, gchar **output);
 
+static void _send_with_priority(IdleConnection *conn, const gchar *msg, guint priority);
+
 static void idle_connection_init(IdleConnection *obj) {
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(obj);
 
@@ -218,6 +232,8 @@ static GObject *idle_connection_constructor(GType type, guint n_params, GObjectC
 
 	self->parser = g_object_new(IDLE_TYPE_PARSER, "connection", self, NULL);
 
+	idle_contact_info_init(self);
+
 	return (GObject *) self;
 }
 
@@ -259,6 +275,10 @@ static void idle_connection_set_property(GObject *obj, guint prop_id, const GVal
 			priv->charset = g_value_dup_string(value);
 			break;
 
+		case PROP_KEEPALIVE_INTERVAL:
+			priv->keepalive_interval = g_value_get_uint(value);
+			break;
+
 		case PROP_QUITMESSAGE:
 			g_free(priv->quit_message);
 			priv->quit_message = g_value_dup_string(value);
@@ -310,6 +330,10 @@ static void idle_connection_get_property(GObject *obj, guint prop_id, GValue *va
 			g_value_set_string(value, priv->charset);
 			break;
 
+		case PROP_KEEPALIVE_INTERVAL:
+			g_value_set_uint(value, priv->keepalive_interval);
+			break;
+
 		case PROP_QUITMESSAGE:
 			g_value_set_string(value, priv->quit_message);
 			break;
@@ -337,6 +361,11 @@ static void idle_connection_dispose (GObject *object) {
 
 	priv->dispose_has_run = TRUE;
 
+	if (priv->keepalive_timeout) {
+		g_source_remove(priv->keepalive_timeout);
+		priv->keepalive_timeout = 0;
+	}
+
 	if (priv->msg_queue_timeout)
 		g_source_remove(priv->msg_queue_timeout);
 
@@ -361,6 +390,8 @@ static void idle_connection_finalize (GObject *object) {
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(object);
 	IdleOutputPendingMsg *msg;
 
+	idle_contact_info_finalize(object);
+
 	g_free(priv->nickname);
 	g_free(priv->server);
 	g_free(priv->password);
@@ -380,6 +411,7 @@ static void idle_connection_finalize (GObject *object) {
 
 static const gchar * interfaces_always_present[] = {
 	TP_IFACE_CONNECTION_INTERFACE_ALIASING,
+	TP_IFACE_CONNECTION_INTERFACE_CONTACT_INFO,
 	IDLE_IFACE_CONNECTION_INTERFACE_RENAMING,
 	TP_IFACE_CONNECTION_INTERFACE_REQUESTS,
 	NULL};
@@ -434,6 +466,9 @@ static void idle_connection_class_init(IdleConnectionClass *klass) {
 	param_spec = g_param_spec_string("charset", "Character set", "The character set to use to communicate with the outside world", "NULL", G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT);
 	g_object_class_install_property(object_class, PROP_CHARSET, param_spec);
 
+	param_spec = g_param_spec_uint("keepalive-interval", "Keepalive interval", "Seconds between keepalive packets, or 0 to disable", 0, G_MAXUINT, DEFAULT_KEEPALIVE_INTERVAL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT);
+	g_object_class_install_property(object_class, PROP_KEEPALIVE_INTERVAL, param_spec);
+
 	param_spec = g_param_spec_string("quit-message", "Quit message", "The quit message to send to the server when leaving IRC", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT);
 	g_object_class_install_property(object_class, PROP_QUITMESSAGE, param_spec);
 
@@ -442,6 +477,8 @@ static void idle_connection_class_init(IdleConnectionClass *klass) {
 
 	param_spec = g_param_spec_boolean("password-prompt", "Password prompt", "Whether the connection should pop up a SASL channel if no password is given", FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT);
 	g_object_class_install_property(object_class, PROP_PASSWORD_PROMPT, param_spec);
+
+	idle_contact_info_class_init(klass);
 }
 
 static GPtrArray *_iface_create_channel_managers(TpBaseConnection *self) {
@@ -604,7 +641,7 @@ static void _start_connecting_continue(IdleConnection *conn) {
 
 	if (!priv->username || !priv->username[0]) {
 		g_free(priv->username);
-		priv->username = g_strdup(priv->nickname);
+		priv->username = g_strdup(g_get_user_name());
 	}
 
 	sconn = IDLE_SERVER_CONNECTION_IFACE(g_object_new(connection_type, "host", priv->server, "port", priv->port, NULL));
@@ -627,6 +664,7 @@ static void _start_connecting_continue(IdleConnection *conn) {
 
 	g_signal_connect(sconn, "received", (GCallback)(sconn_received_cb), conn);
 
+	idle_parser_add_handler(conn->parser, IDLE_PARSER_CMD_ERROR, _error_handler, conn);
 	idle_parser_add_handler(conn->parser, IDLE_PARSER_NUMERIC_ERRONEOUSNICKNAME, _erroneous_nickname_handler, conn);
 	idle_parser_add_handler(conn->parser, IDLE_PARSER_NUMERIC_NICKNAMEINUSE, _nickname_in_use_handler, conn);
 	idle_parser_add_handler(conn->parser, IDLE_PARSER_NUMERIC_WELCOME, _welcome_handler, conn);
@@ -640,6 +678,7 @@ static void _start_connecting_continue(IdleConnection *conn) {
 	irc_handshakes(conn);
 }
 
+static gboolean keepalive_timeout_cb(gpointer user_data);
 static gboolean msg_queue_timeout_cb(gpointer user_data);
 
 static void sconn_status_changed_cb(IdleServerConnectionIface *sconn, IdleServerConnectionState state, IdleServerConnectionStateReason reason, IdleConnection *conn) {
@@ -687,6 +726,9 @@ static void sconn_status_changed_cb(IdleServerConnectionIface *sconn, IdleServer
 			break;
 
 		case SERVER_CONNECTION_STATE_CONNECTED:
+			if (priv->keepalive_interval != 0 && priv->keepalive_timeout == 0)
+				priv->keepalive_timeout = g_timeout_add_seconds(priv->keepalive_interval, keepalive_timeout_cb, conn);
+
 			if ((priv->msg_queue_timeout == 0) && (g_queue_get_length(priv->msg_queue) > 0)) {
 				IDLE_DEBUG("we had messages in queue, start unloading them now");
 
@@ -711,6 +753,24 @@ static void sconn_received_cb(IdleServerConnectionIface *sconn, gchar *raw_msg,
 	g_free(converted);
 }
 
+static gboolean keepalive_timeout_cb(gpointer user_data) {
+	IdleConnection *conn = IDLE_CONNECTION(user_data);
+	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
+	gchar cmd[IRC_MSG_MAXLEN + 1];
+	gint64 ping_time;
+
+	if (priv->sconn_status != SERVER_CONNECTION_STATE_CONNECTED) {
+		priv->keepalive_timeout = 0;
+		return FALSE;
+	}
+
+	ping_time = g_get_real_time();
+	g_snprintf(cmd, IRC_MSG_MAXLEN + 1, "PING %" G_GINT64_FORMAT, ping_time);
+	_send_with_priority(conn, cmd, SERVER_CMD_MIN_PRIORITY);
+
+	return TRUE;
+}
+
 static gboolean msg_queue_timeout_cb(gpointer user_data) {
 	IdleConnection *conn = IDLE_CONNECTION(user_data);
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
@@ -842,6 +902,55 @@ idle_connection_get_max_message_length(IdleConnection *conn)
 	return IRC_MSG_MAXLEN - 100;
 }
 
+static IdleParserHandlerResult _error_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data) {
+	IdleConnection *conn = IDLE_CONNECTION(user_data);
+	GHashTable *details = NULL;
+	TpConnectionStatus status = conn->parent.status;
+	TpConnectionStatusReason reason;
+	const gchar *msg;
+	const gchar *begin;
+	const gchar *end;
+	const gchar *error_name;
+
+	switch (status) {
+		case TP_CONNECTION_STATUS_CONNECTING:
+			reason = TP_CONNECTION_STATUS_REASON_AUTHENTICATION_FAILED;
+			error_name = TP_ERROR_STR_AUTHENTICATION_FAILED;
+			break;
+
+		case TP_CONNECTION_STATUS_CONNECTED:
+			reason = TP_CONNECTION_STATUS_REASON_NETWORK_ERROR;
+			error_name = TP_ERROR_STR_NETWORK_ERROR;
+			break;
+
+		default:
+			return IDLE_PARSER_HANDLER_RESULT_HANDLED;
+	}
+
+	msg = g_value_get_string(g_value_array_get_nth(args, 0));
+	begin = strchr(msg, '(');
+	end = strrchr(msg, ')');
+
+	if (begin != NULL && end != NULL && begin < end - 1) {
+		gchar *server_msg;
+		guint length;
+
+		begin++;
+		end--;
+		length = end - begin + 1;
+		server_msg = g_strndup(begin, length);
+		details = tp_asv_new ("server-message", G_TYPE_STRING, server_msg, NULL);
+		g_free(server_msg);
+	}
+
+	tp_base_connection_disconnect_with_dbus_error(TP_BASE_CONNECTION(conn), error_name, details, reason);
+
+	if (details != NULL)
+		g_hash_table_unref(details);
+
+	return IDLE_PARSER_HANDLER_RESULT_HANDLED;
+}
+
 static IdleParserHandlerResult _erroneous_nickname_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data) {
 	IdleConnection *conn = IDLE_CONNECTION(user_data);
 
@@ -936,8 +1045,6 @@ _whois_user_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray
 			const char* host = g_value_get_string(g_value_array_get_nth(args, 2));
 			priv->relay_prefix = g_strdup_printf("%s!%s@%s", priv->nickname, user, host);
 			IDLE_DEBUG("user host prefix = %s", priv->relay_prefix);
-
-			return IDLE_PARSER_HANDLER_RESULT_HANDLED;
 	}
 	return IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED;
 }
