@@ -419,6 +419,10 @@ static void idle_muc_channel_get_property(GObject *object, guint property_id, GV
 					TP_IFACE_CHANNEL, "InitiatorHandle",
 					TP_IFACE_CHANNEL, "InitiatorID",
 					TP_IFACE_CHANNEL, "Requested",
+					TP_IFACE_CHANNEL_INTERFACE_MESSAGES, "MessagePartSupportFlags",
+					TP_IFACE_CHANNEL_INTERFACE_MESSAGES, "DeliveryReportingSupport",
+					TP_IFACE_CHANNEL_INTERFACE_MESSAGES, "SupportedContentTypes",
+					TP_IFACE_CHANNEL_INTERFACE_MESSAGES, "MessageTypes",
 					NULL);
 			g_value_take_boxed (value, props);
 			break;
@@ -1135,7 +1139,10 @@ void idle_muc_channel_join(IdleMUCChannel *chan, TpHandle joiner) {
 		/* woot we managed to get into a channel, great */
 		change_state(chan, MUC_STATE_JOINED);
 		tp_group_mixin_change_members((GObject *)(chan), NULL, set, NULL, NULL, NULL, joiner, TP_CHANNEL_GROUP_CHANGE_REASON_NONE);
-		tp_group_mixin_change_flags((GObject *)(chan), TP_CHANNEL_GROUP_FLAG_CAN_ADD, 0);
+		tp_group_mixin_change_flags((GObject *)(chan),
+			TP_CHANNEL_GROUP_FLAG_CAN_ADD |
+			TP_CHANNEL_GROUP_FLAG_MESSAGE_DEPART,
+			0);
 
 		send_mode_query_request(chan);
 
@@ -1669,7 +1676,7 @@ static gboolean send_kick_request(IdleMUCChannel *obj, TpHandle handle, const gc
 	}
 
 	if (msg != NULL) {
-		g_snprintf(cmd, IRC_MSG_MAXLEN + 1, "KICK %s %s %s", priv->channel_name, nick, msg);
+		g_snprintf(cmd, IRC_MSG_MAXLEN + 1, "KICK %s %s :%s", priv->channel_name, nick, msg);
 	} else {
 		g_snprintf(cmd, IRC_MSG_MAXLEN + 1, "KICK %s %s", priv->channel_name, nick);
 	}
@@ -1737,7 +1744,7 @@ static void part_from_channel(IdleMUCChannel *obj, const gchar *msg) {
 	priv = IDLE_MUC_CHANNEL_GET_PRIVATE(obj);
 
 	if (msg != NULL) {
-		g_snprintf(cmd, IRC_MSG_MAXLEN + 1, "PART %s %s", priv->channel_name, msg);
+		g_snprintf(cmd, IRC_MSG_MAXLEN + 1, "PART %s :%s", priv->channel_name, msg);
 	} else {
 		g_snprintf(cmd, IRC_MSG_MAXLEN + 1, "PART %s", priv->channel_name);
 	}
@@ -2361,10 +2368,43 @@ static void idle_muc_channel_set_properties (TpSvcPropertiesInterface *iface, co
 
 gboolean idle_muc_channel_is_modechar(char c) {
 	switch (c) {
-		case '@':
-		case '&':
-		case '+':
+		/* founder */
+		case '*':
 		case '~':
+
+		/* admin */
+		case '!':
+		case '&':
+
+		/* chanop */
+		case '@':
+
+		/* halfop */
+		case '%':
+
+		/* voice */
+		case '+':
+			return TRUE;
+
+		default:
+			return FALSE;
+	}
+}
+
+gboolean idle_muc_channel_is_typechar(gchar c)
+{
+	switch (c) {
+		/* standard channel */
+		case '#':
+
+		/* local to a server */
+		case '&':
+
+		/* no support for modes */
+		case '+':
+
+		/* safe channel */
+		case '!':
 			return TRUE;
 
 		default:
